const { getOptions } = require("loader-utils");
const path = require("path");
const fsExtra = require("fs-extra-promise");
const hasha = require("hasha");

function getMd5(file)
{
    return hasha.fromStream(fsExtra.createReadStream(file), { algorithm: "md5" });
}

function createError(message)
{
    return new Error("copy-node-loader: " + message);
}

async function loader(content)
{
    if(!(content instanceof Buffer))
        throw createError("content is not");
    let options = getOptions(this);

    if(!options || !options.outputPath)
        throw createError("path is undefined");
    if(!options || !options.publicPath)
        throw createError("public path is undefined");

    if(path.isAbsolute(options.publicPath))
        console.warn("public path is absolute path");
    let name = options.name;
    let nodeExt = path.extname(this.resourcePath);
    let nodeName = path.basename(this.resourcePath, nodeExt);
    if(nodeExt.length > 1)
        nodeExt = nodeExt.substr(1);

    if(name.indexOf("[hash]") != -1)
    {
        let hashCode = await getMd5(this.resourcePath);
        name = name.replace(/\[hash\]/g, hashCode);
    }
    name = name.replace(/\[name\]/, nodeName);
    name = name.replace(/\[ext\]/, nodeExt);

    let relativePath = path.join(options.publicPath, name);
    if(path.isAbsolute(relativePath))
        console.warn("copy-node-loader: warnning, The publicPath is absolute path(" + relativePath + ")");
    await fsExtra.ensureDirAsync(options.outputPath);
    await fsExtra.writeFileAsync(path.resolve(options.outputPath, name), content);

    return "try {global.process.dlopen(module, " + JSON.stringify(relativePath) + "); } catch(e) {" +
        "throw new Error('Cannot open ' + " + JSON.stringify(relativePath) + " + ': ' + e);}";
}

/**
 * Options
 * name: string include [name] [hash] [ext] keywords
 * outputPath: string
 * publicPath: string
 */
module.exports = function (content) {
    let callback = this.async();
    loader.call(this, content)
        .then((content)=>{
            callback(null, content);
        })
        .catch((err)=>{
            callback(err);
        });
}

module.exports.raw = true;